import csv
import matplotlib.pyplot as plt


def read_data(file_name):
    """Reads minimal number of modes to hit given performance target with a given number of repeaters.

    Assumes the first column is the target, the last one is the number of the repeaters and the second is the minimal
    number of modes.
    """
    modes = []
    repeaters = []
    with open(file_name) as file:
        reader = csv.reader(file, delimiter=',')
        for i, row in enumerate(reader):
            if i == 0 or int(row[-1]) == 0:
                # assumes first row is title row
                # ignores elementary link data in case that exists
                continue
            modes.append(int(row[1]))
            repeaters.append(int(row[-1]))
            target = int(row[0])

    return modes, repeaters, target


def prepare_data(file_names):
    """Reads data from a list of files and arranges it into a list of lists.
    """
    modes = [[], [], []]
    repeaters = [[], [], []]
    targets = [0, 0, 0]
    for i, file in enumerate(file_names):
        mode, repeater, target = read_data(file_name=file)
        modes[i] = mode
        repeaters[i] = repeater
        targets[i] = target

    return modes, repeaters, targets


def scatter_plot(modes, repeaters, targets):
    fig = plt.figure()
    ax1 = fig.add_subplot(111)

    ax1.scatter(repeaters[0], modes[0], s=20, marker="o", label='SKR = {} Hz'.format(int(targets[0])))
    ax1.scatter(repeaters[1], modes[1], s=20, marker="o", label='SKR = {} Hz'.format(int(targets[1])))
    ax1.scatter(repeaters[2], modes[2], s=20, marker="o", label='SKR = {} Hz'.format(int(targets[2])))

    plt.legend(loc='upper right')
    plt.show()


def scatter_plot_broken_axis_one_target(modes, repeaters, target, bounds):
    x_cm = 8.
    y_cm = 5.
    f, (ax1, ax2, ax3) = plt.subplots(3, 1, sharex=True, figsize=(x_cm / 2.54, y_cm / 2.54))
    ax1.scatter(repeaters, modes, s=8, label='SKR = {} Hz'.format(int(target)))
    ax2.scatter(repeaters, modes, s=8, label='SKR = {} Hz'.format(int(target)))
    ax3.scatter(repeaters, modes, s=8, label='SKR = {} Hz'.format(int(target)))

    # zoom-in / limit the view to different portions of the data
    # the numbers need to be picked manually to get something that looks good
    ax1.set_ylim(bounds[0][0], bounds[0][1])  # outliers
    ax2.set_ylim(bounds[1][0], bounds[1][1])  # more outliers
    ax3.set_ylim(bounds[2][0], bounds[2][1])        # most of the data

    # Break and put lines between ax 1 and ax 2

    ax1.spines['bottom'].set_visible(False)
    ax2.spines['top'].set_visible(False)
    ax1.xaxis.tick_top()
    ax1.tick_params(labeltop=False)  # don't put tick labels at the top
    ax1.tick_params(top=False)  # don't put tick labels at the top
    ax2.xaxis.tick_bottom()

    d = .01  # how big to make the diagonal lines in axes coordinates
    # arguments to pass to plot, just so we don't keep repeating them
    kwargs = dict(transform=ax1.transAxes, color='k', clip_on=False)
    ax1.plot((-d, +d), (-d, +d), **kwargs)        # top-left diagonal
    ax1.plot((1 - d, 1 + d), (-d, +d), **kwargs)  # top-right diagonal

    kwargs.update(transform=ax2.transAxes)  # switch to the bottom axes
    ax2.plot((-d, +d), (1 - d, 1 + d), **kwargs)  # bottom-left diagonal
    ax2.plot((1 - d, 1 + d), (1 - d, 1 + d), **kwargs)  # bottom-right diagonal

    # Break and put lines between ax 2 and ax 3

    ax2.spines['bottom'].set_visible(False)
    ax3.spines['top'].set_visible(False)
    ax2.xaxis.tick_top()
    ax2.tick_params(top=False)  # don't put ticks at the top as they would show up midplot
    ax3.xaxis.tick_bottom()

    d = .01  # how big to make the diagonal lines in axes coordinates
    # arguments to pass to plot, just so we don't keep repeating them
    kwargs = dict(transform=ax2.transAxes, color='k', clip_on=False)
    ax2.plot((-d, +d), (-d, +d), **kwargs)        # top-left diagonal
    ax2.plot((1 - d, 1 + d), (-d, +d), **kwargs)  # top-right diagonal

    kwargs.update(transform=ax3.transAxes)  # switch to the bottom axes
    ax3.plot((-d, +d), (1 - d, 1 + d), **kwargs)  # bottom-left diagonal
    ax3.plot((1 - d, 1 + d), (1 - d, 1 + d), **kwargs)  # bottom-right diagonal

    ax1.ticklabel_format(scilimits=(0, 0))
    ax1.set_yticks([ax1.get_yticks()[1]])
    ax2.set_yticks([ax2.get_yticks()[1]])
    ax3.set_yticks([ax3.get_yticks()[0], ax3.get_yticks()[1]])

    ax3.set_xticks(range(2, 8))

    ax1.tick_params(axis="both", labelsize=8)
    ax2.tick_params(axis="both", labelsize=8)
    ax3.tick_params(axis="both", labelsize=8)

    return ax1, ax2, ax3


def scatter_plot_broken_axis(modes, repeaters, targets):
    # These need to be tinkered with manually to get a good-looking plot
    bounds_10_Hz = [[350000, 450000], [4500, 5500], [1, 400]]
    ax1, ax2, ax3, = scatter_plot_broken_axis_one_target(modes=modes[1], repeaters=repeaters[1],
                                                         target=targets[1], bounds=bounds_10_Hz)
    ax2.set_ylabel("Minimal #Modes")
    plt.xlabel("#Repeaters")
    plt.savefig("testing.svg", dpi=300, bbox_inches="tight")
    plt.savefig("testing.png", dpi=300, bbox_inches="tight")


def scatter_plot_logscale(modes, repeaters, targets):
    """This function produces a scatter plot of the minimal number of modes required to hit given performance targets
    for different number of repeaters placed symmetrically in a fiber path with the same length and attenuation as the
    Bonn - Berlin path. The y-axis, corresponding to the number of modes, is in logarithmic scale.

    Parameters
    ----------
    modes : list
        List of lists with number of modes required for each repeater and target.
    repeaters : list
        List of list with numbers of repeaters for each target.
    targets : list
        List of SKR performance targets.
    """

    x_cm = 8.
    y_cm = 5.
    fig = plt.figure(figsize=(x_cm / 2.54, y_cm / 2.54))
    ax = fig.add_subplot(111)

    ax.grid(which="both", alpha=0.5)
    # Ensures data points are drawn above grid
    ax.set_axisbelow(True)
    markers = ['o', 'x', '^']
    for i in range(len(modes)):
        ax.scatter(repeaters[i], modes[i], marker=markers[i], s=8, label='SKR = {} Hz'.format(int(targets[i])))

    ax.set_yscale("log")
    ax.set_xticks(repeaters[0])
    # These are picked manually to ensure a plot that's not too cramped but still contains useful information
    ax.set_yticks([1, 100, 10000, 1e6, 1e8, 1e10])

    plt.ylabel("Minimal #Modes")
    plt.xlabel("#Repeaters")
    plt.legend(loc='upper right')
    plt.savefig("../../Figures/minimal_number_modes_logscale.png", dpi=300, bbox_inches="tight")


def produce_number_of_modes_plot(file_names=["data/1_Hz_symmetric_chain_Bonn_Berlin_performance.csv",
                                             "data/10_Hz_symmetric_chain_Bonn_Berlin_performance.csv",
                                             "data/100_Hz_symmetric_chain_Bonn_Berlin_performance.csv"]):
    modes, repeaters, targets = prepare_data(file_names=file_names)
    scatter_plot_logscale(modes, repeaters, targets)


if __name__ == "__main__":
    produce_number_of_modes_plot()
