import os
import matplotlib
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import yaml
from math import pi
from netsquid_netconf.netconf import Loader
from argparse import ArgumentParser

font = {'size': 12}

matplotlib.rc('font', **font)

NAME2DISPLAY = {"fidelity": "link \n fidelity",
                "swap_quality": "swap quality",
                "coherence_time": "coherence time",
                "efficiency": "efficiency",
                "num_modes": "# modes"}

TO_PROB_NO_ERROR_FUNCTION = {"efficiency": lambda x: x,
                             "num_modes": lambda x, fiber_data_file="data/Berlin_Bonn_fiber_data.csv":
                             compute_success_prob_from_num_modes(x, fiber_data_file),
                             "coherence_time": lambda x: np.exp(-1 * (1 / x)**2),
                             "swap_quality": lambda x: x,
                             "fidelity": lambda x: x,
                             }


def plot_circular(improvement_dictionary_1, improvement_dictionary_2, improvement_dictionary_3, max_radius=None,
                  baseline_name="baseline", solution_name_1="improved_1", solution_name_2="improved_2",
                  solution_name_3="improved_3", label_1=None, label_2=None, label_3=None, output_file="circular_plot",
                  log=False):
    angles_1 = [n / float(len(improvement_dictionary_1)) * 2 * pi for n in range(len(improvement_dictionary_1))]
    r_1 = list(improvement_dictionary_1.values())
    angles_2 = [n / float(len(improvement_dictionary_2)) * 2 * pi for n in range(len(improvement_dictionary_2))]
    r_2 = list(improvement_dictionary_2.values())
    angles_3 = [n / float(len(improvement_dictionary_3)) * 2 * pi for n in range(len(improvement_dictionary_3))]
    r_3 = list(improvement_dictionary_3.values())

    ax = plt.subplot(111, polar=True)
    ax.set_axisbelow(False)
    if label_1 is None:
        label_1 = solution_name_1
    if label_2 is None:
        label_2 = solution_name_2
    if label_3 is None:
        label_3 = solution_name_3

    ax.plot(angles_1 + [angles_1[0]], r_1 + [r_1[0]], linewidth=2.5, label=label_1)
    ax.plot(angles_2 + [angles_2[0]], r_2 + [r_2[0]], linewidth=2.5, label=label_2, linestyle='dashed')
    ax.plot(angles_3 + [angles_3[0]], r_3 + [r_3[0]], linewidth=2.5, label=label_3, linestyle='dotted')

    if max_radius is not None:
        ax.set_rmax(max_radius)
        if max_radius > 10:
            ticks = [round(tick / 5) * 5 for tick in np.arange(0, max_radius, max_radius / 4)]
            ax.set_rticks(ticks)
        else:
            ax.set_rticks(np.arange(1, max_radius, 2))
    ax.set_theta_offset(pi / 2)
    ax.set_theta_direction(-1)
    plt.xticks(angles_1)
    angle_labels = [NAME2DISPLAY[param_name] for param_name in improvement_dictionary_1.keys()]
    ax.set_xticklabels(angle_labels)
    ax.xaxis.set_tick_params(pad=25)
    ax.set_rlabel_position(0)
    ax.grid(True)
    if log:
        ax.set_rscale('symlog')
    plt.legend(loc=(-0.2, 1))

    # Save
    # plt.savefig(os.path.join("../../Figures", output_file + '.svg'), dpi=300, bbox_inches='tight')
    plt.savefig(os.path.join("../../Figures", output_file + '.png'), dpi=300, bbox_inches='tight')


def compute_improvement_factor_dict(baseline_dict, solution_dict):
    improvement_factor_dict = {}
    for parameter, value in baseline_dict.items():
        if parameter == "coherence_time":
            improvement_factor_dict[parameter] = (solution_dict[parameter] / value) ** 2
        else:
            baseline_prob_no_error = TO_PROB_NO_ERROR_FUNCTION[parameter](value)
            prob_no_error = TO_PROB_NO_ERROR_FUNCTION[parameter](solution_dict[parameter])
            improvement_factor_dict[parameter] = 1 / (np.log(prob_no_error) / np.log(baseline_prob_no_error))
    return improvement_factor_dict


def read_param_values(param_names, parameter_file):
    with open(parameter_file, "r") as stream:
        parameter_dict = yaml.load(stream, Loader=Loader)

    return {param_name: parameter_dict[param_name] for param_name in param_names}


def average_total_attenuation_from_file(file):
    """Computes total average attenuation of fiber data held in `file`.

    Assumes that the data is stored in a csv format with one column being named `attenuation_coefficient` and another
    `length`. We double the average length because we are interested in the average length of elementary links rather
    than the distance between nodes and heralding stations.

    Parameters
    ----------
    file : str
        Name of file storing fiber data.

    Returns
    -------
    float
        Average of total attenuation.

    """
    df = pd.read_csv(file)
    return np.average(df["attenuation_coefficient"] * df["length"] * 2)


def compute_success_prob_from_num_modes(num_modes, fiber_data_file="Berlin_Bonn_fiber_data.csv"):
    """Computes photon success probability given a number of modes `n_modes` and a baseline survival probability.

    The baseline survival probability is computed from the average attenuation attenuation of the fiber segments defined
    in the `fiber_data_file`.

    Parameters
    ----------
    num_modes : int
        Number of multiplexing modes.
    fiber_data_file : str
        Name of file storing fiber data.

    Returns
    -------
    float
        Multiplexed photon survival probability.

    """

    average_total_attenuation = average_total_attenuation_from_file(fiber_data_file)
    baseline_survival_probability = 10 ** (- average_total_attenuation / 10)

    return 1 - (1 - baseline_survival_probability) ** num_modes


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("--parameters", nargs="+", default=None, required=True,
                        help="Parameters to include in circular plot.")
    parser.add_argument("--baseline_param_file", type=str, required=True, help="Name of baseline parameter file.")
    parser.add_argument("--solution_param_file_1", type=str, required=True, help="Name of 1st solution parameter file.")
    parser.add_argument("--solution_param_file_2", type=str, required=True, help="Name of 2nd solution parameter file.")
    parser.add_argument("--solution_param_file_3", type=str, required=True, help="Name of 3rd solution parameter file.")
    parser.add_argument("--label_1", type=str, required=False, default=None, help="Label for 1st solution.")
    parser.add_argument("--label_2", type=str, required=False, default=None, help="Label for 2nd solution.")
    parser.add_argument("--label_3", type=str, required=False, default=None, help="Label for 3rd solution.")
    parser.add_argument("--output_file", type=str, required=False, default="minimal_requirements_different_skr_targets",
                        help="Where to save plot.")
    parser.add_argument('--log', action='store_true', help="Whether or not to use logarithmic scale.")
    parser.set_defaults(log=True)

    args, unknown = parser.parse_known_args()

    baseline_param_dict = read_param_values(args.parameters, args.baseline_param_file)
    solution_param_dict_1 = read_param_values(args.parameters, args.solution_param_file_1)
    solution_param_dict_2 = read_param_values(args.parameters, args.solution_param_file_2)
    solution_param_dict_3 = read_param_values(args.parameters, args.solution_param_file_3)

    imp_factor_dict_1 = compute_improvement_factor_dict(baseline_param_dict, solution_param_dict_1)
    imp_factor_dict_2 = compute_improvement_factor_dict(baseline_param_dict, solution_param_dict_2)
    imp_factor_dict_3 = compute_improvement_factor_dict(baseline_param_dict, solution_param_dict_3)

    max_radius = max([max(imp_factor_dict_1.values()), max(imp_factor_dict_2.values())])
    plot_circular(improvement_dictionary_1=imp_factor_dict_1,
                  improvement_dictionary_2=imp_factor_dict_2,
                  improvement_dictionary_3=imp_factor_dict_3,
                  max_radius=max_radius,
                  baseline_name=args.baseline_param_file.split(".yaml")[0],
                  solution_name_1=args.solution_param_file_1.split(".yaml")[0],
                  solution_name_2=args.solution_param_file_2.split(".yaml")[0],
                  solution_name_3=args.solution_param_file_3.split(".yaml")[0],
                  label_1=args.label_1,
                  label_2=args.label_2,
                  label_3=args.label_3,
                  output_file=args.output_file,
                  log=args.log)
