from argparse import ArgumentParser
import csv
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib

matplotlib.rcParams.update({'font.size': 8})


def prepare_data(diff_targets, file_names):
    """Reads data from a list of files and arranges it into a list of lists.
    """
    df_list = read_processed_data(diff_targets, file_names)
    rep_list = []
    inv_qber_x_list = []
    inv_qber_z_list = []
    rate_list = []
    inv_qber_x_list_err = []
    inv_qber_z_list_err = []
    rate_list_err = []

    for df in df_list:
        df["inv_qber_x"] = 1 - df["Qber_x"]
        df["inv_qber_z"] = 1 - df["Qber_z"]
        df["rate"] = 1. / df["duration_per_success"]
        df["rate_error"] = df["duration_per_success_error"] / df["duration_per_success"] ** 2
        if not diff_targets:
            rep_list.append(int(df['n_repeaters']))
        inv_qber_x_list.append(float(df['inv_qber_x']))
        inv_qber_z_list.append(float(df['inv_qber_z']))
        rate_list.append(float(df['rate']))
        inv_qber_x_list_err.append(float(df['Qber_x_error']))
        inv_qber_z_list_err.append(float(df['Qber_z_error']))
        rate_list_err.append(float(df['rate_error']))
    return rep_list, inv_qber_x_list, inv_qber_z_list, rate_list, inv_qber_x_list_err, inv_qber_z_list_err, rate_list_err


def read_processed_data(diff_targets, file_names):
    df_list = []
    for file in file_names:
        new_df = pd.read_csv(file)
        if not diff_targets:
            new_df["n_repeaters"] = int(file.split('_repeater')[0].split('best_')[1])
        df_list.append(new_df)
    return df_list


def scatter_plot(rep_list, inv_qber_x_list, inv_qber_z_list, rate_list, inv_qber_x_list_err,
                 inv_qber_z_list_err, rate_list_err, diff_targets):
    """
    """

    x_cm = 8.
    y_cm = 5.
    fig = plt.figure(figsize=(x_cm / 2.54, y_cm / 2.54))
    ax = fig.add_subplot(111)
    if diff_targets:
        rep_list = [1, 10, 100]

    ax.errorbar(rep_list, inv_qber_x_list, fmt='o', color='orange',
                yerr=inv_qber_x_list_err, label="QBER", alpha=0.5)
    # ax.errorbar(rep_list, inv_qber_z_list, fmt='x', color='orange',
    #            yerr=inv_qber_z_list_err, label="QBER Z", alpha=0.5)
    ax.set_ylabel("1 - QBER")
    ax.yaxis.label.set_color("orange")
    ax.tick_params(axis="y", colors="orange")

    ax2 = ax.twinx()
    ax2.errorbar(rep_list, rate_list, fmt='^', color='blue',
                 yerr=rate_list_err, label="Rate (Hz)", alpha=0.5)
    ax2.set_ylabel("Entanglement \n generation rate (Hz)")
    ax2.spines["right"].set_color("blue")
    ax2.spines["left"].set_color("orange")
    ax2.yaxis.label.set_color("blue")
    ax2.tick_params(axis="y", colors="blue")
    # ask matplotlib for the plotted objects and their labels
    lines, labels = ax.get_legend_handles_labels()
    lines2, labels2 = ax2.get_legend_handles_labels()
    # ax2.legend(lines + lines2, labels + labels2, loc="center")
    fig.legend(lines + lines2, labels + labels2, bbox_to_anchor=(0.9, 0.), loc="center left")
    if diff_targets:
        # ax2.set_xticks([1, 10, 100])
        # ax2.set_xticklabels(["1", "10", "100"])
        ax2.set_xticks([10, 100])
        ax2.set_xticklabels(["10", "100"])
        ax2.set_xscale("log")
        ax.set_xlabel("SKR (Hz)")
        # plt.savefig("../../Figures/qber_rate_targets.svg", dpi=300, bbox_inches="tight")
        plt.savefig("../../Figures/qber_rate_targets", dpi=300, bbox_inches="tight")
    else:
        ax2.set_xticks([4, 5, 6, 7])
        ax2.set_xticklabels(["4", "5", "6", "7"])
        ax.set_xlabel("#Repeaters")
        # plt.savefig("../../Figures/qber_rate.svg", dpi=300, bbox_inches="tight")
        plt.savefig("../../Figures/qber_rate", dpi=300, bbox_inches="tight")


def produce_qber_rate_plot(diff_targets):
    if diff_targets:
        file_names = ["data/best_1_Hz_solution_output.csv",
                      "data/best_free_repeaters_solution_config_output.csv",
                      "data/best_100_Hz_solution_output.csv"]
    else:
        file_names = ["data/best_4_repeater_solution_output.csv",
                      "data/best_5_repeater_solution_output.csv",
                      "data/best_6_repeater_solution_output.csv",
                      "data/best_7_repeater_solution_output.csv"]
    rep_list, inv_qber_x_list, inv_qber_z_list, rate_list, inv_qber_x_list_err, inv_qber_z_list_err, rate_list_err \
        = prepare_data(file_names=file_names, diff_targets=diff_targets)
    scatter_plot(rep_list, inv_qber_x_list, inv_qber_z_list, rate_list,
                 inv_qber_x_list_err, inv_qber_z_list_err, rate_list_err, diff_targets)


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument('--diff_targets', action='store_true', help="""If True, produces plot for different SKR targets.
Otherwise, produces it for different number of repeaters.""")
    parser.set_defaults(diff_targets=False)
    args, unknown = parser.parse_known_args()

    produce_qber_rate_plot(args.diff_targets)
